package in.mobiocean.mo_mifare_plugin;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.telpo.tps550.api.TelpoException;
import com.telpo.tps550.api.nfc.Nfc;
import com.telpo.tps550.api.util.StringUtil;

/** MoMifarePlugin */
public class MoMifarePlugin implements FlutterPlugin, MethodCallHandler, ActivityAware, ReadCallback {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  private Activity activity;
  private Context context;
  private NfcSample nfcSample;
  private Result readResult;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "mo_mifare_plugin");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding){
    activity = binding.getActivity();
    context = binding.getActivity();
    nfcSample = new NfcSample(activity, this);
  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding){
    activity = binding.getActivity();
    context = binding.getActivity();
    if(nfcSample == null)
      nfcSample = new NfcSample(activity, this);
    else
      nfcSample.updateCallBack(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    readResult = null;
    if (call.method.equals("startNfc")) {
      int res = nfcSample.startNfc();
      result.success(""+res);
    } else if (call.method.equals("closeNfc")) {
      int res = nfcSample.closeNfc();
      result.success(""+res);
    } else if (call.method.equals("readCard")) {
      readResult = result;
      nfcSample.readCard();
    } else if (call.method.equals("haltOperation")) {
      int res = nfcSample.m1HaltOperation();
      result.success(""+res);
    } else if (call.method.equals("authBlock")) {
      byte blockNo = call.argument("blockNo");
      byte[] password = call.argument("password");
      int res = nfcSample.m1CardAuthenticate(blockNo, password);
      result.success(""+res);
    } else if (call.method.equals("read")) {
      byte blockNo = call.argument("blockNo");
      String res = nfcSample.readValueData(blockNo);
      result.success(""+res);
    } else if (call.method.equals("write")) {
      byte blockNo = call.argument("blockNo");
      String conent = call.argument("conent");
      int res = nfcSample.writeValueData(blockNo, conent);
      result.success(""+res);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    context = null;
    activity = null;
    readResult = null;
  }

  @Override
  public void onDetachedFromActivity() {
    context = null;
    activity = null;
    readResult = null;
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    readResult = null;
  }

  @Override
  public void timeout() {
    if(readResult != null)
    readResult.success("" + -1);
  }

  @Override
  public void unknownCard() {
    if(readResult != null)
    readResult.success("" + -2);
  }

  @Override
  public void cardRead(String atqa, String sak, String uid) {
    if(readResult != null)
    readResult.success("{" +
            "\"atqa\":\"" + atqa + "\"" +
            "\"sak\":\"" + sak + "\"" +
            "\"uid\":\"" + uid + "\"" +
            "}");
  }

  @Override
  public void readFailure(Exception e) {
    if(readResult != null)
    readResult.success(""+-3);
  }
}
